var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import React, { forwardRef, useEffect, useMemo, useRef, useState } from 'react';
import debounce from 'lodash.debounce';
import { monaco, createEditor, getMonacoLanguage, updateEditorKeybindings, registerEditorOpenHandler } from '@codingame/monaco-editor-wrapper';
import { useDeepMemo, useLastValueRef, useLastVersion, useThemeData, useUserConfiguration } from './hooks';
import './style';
var STATUS_BAR_HEIGHT = 20;
function fixCode(code) {
    // If monaco editor is provided code using \r\n
    // The editor seems to flicker and the language server is spammed by document/didChange events
    // TODO investigate me?
    return code.replace(/\r\n?/g, '\n');
}
// Some editor properties are managed directly by monaco-vim / monaco-emacs so we don't want to override them
var keyBindingsManagedOptions = {
    vim: new Set(['cursorBlinking', 'cursorWidth']),
    emacs: new Set(['cursorBlinking', 'cursorStyle'])
};
function removeKeyBindingsManagedOptions(options, keyBindingsMode) {
    var managedOptions = keyBindingsManagedOptions[keyBindingsMode];
    if (managedOptions == null) {
        return options;
    }
    else {
        return Object.fromEntries(Object.entries(options)
            .filter(function (_a) {
            var optionId = _a[0];
            return !managedOptions.has(optionId);
        }));
    }
}
var viewStates = new Map();
export function defaultRestoreViewState(editor, model) {
    var viewState = viewStates.get(model.uri.toString());
    if (viewState != null) {
        editor.restoreViewState(viewState);
    }
}
export function defaultSaveViewState(editor, model) {
    var viewState = editor.saveViewState();
    var key = model.uri.toString();
    if (viewState != null) {
        viewStates.set(key, viewState);
    }
    else {
        viewStates.delete(key);
    }
}
function MonacoEditor(_a, ref) {
    var _this = this;
    var _b = _a.height, requestedHeight = _b === void 0 ? '100%' : _b, maxHeight = _a.maxHeight, _c = _a.minHeight, minHeight = _c === void 0 ? 350 : _c, fileUri = _a.fileUri, programmingLanguage = _a.programmingLanguage, value = _a.value, options = _a.options, overrideServices = _a.overrideServices, onChange = _a.onChange, keyBindings = _a.keyBindings, _d = _a.keyBindingsMode, keyBindingsMode = _d === void 0 ? 'classic' : _d, markers = _a.markers, _e = _a.saveViewState, saveViewState = _e === void 0 ? defaultSaveViewState : _e, _f = _a.restoreViewState, restoreViewState = _f === void 0 ? defaultRestoreViewState : _f, onEditorOpenRequest = _a.onEditorOpenRequest;
    var editorRef = useRef();
    var modelRef = useRef();
    var preventTriggerChangeEventRef = useRef(false);
    var _g = useState(requestedHeight !== 'auto' ? requestedHeight : 50), height = _g[0], setHeight = _g[1];
    var themeData = useThemeData();
    var containerRef = useRef(null);
    var statusBarRef = useRef(null);
    var monacoLanguage = useMemo(function () { return programmingLanguage != null ? getMonacoLanguage(programmingLanguage) : undefined; }, [programmingLanguage]);
    var userConfiguration = useUserConfiguration(monacoLanguage);
    var memoizedOptions = useDeepMemo(function () { return options; }, [options]);
    var allOptions = useMemo(function () {
        return removeKeyBindingsManagedOptions(__assign(__assign(__assign({}, userConfiguration), memoizedOptions), { automaticLayout: true }), keyBindingsMode);
    }, [memoizedOptions, userConfiguration, keyBindingsMode]);
    var modelUri = useMemo(function () {
        return fileUri != null ? monaco.Uri.parse(fileUri) : undefined;
    }, [fileUri]);
    var fixedCode = useMemo(function () { return value != null ? fixCode(value) : null; }, [value]);
    var valueRef = useLastValueRef(fixedCode);
    var lastSaveViewState = useLastVersion(saveViewState);
    var lastRestoreViewState = useLastVersion(restoreViewState);
    // Create/Update model
    useEffect(function () {
        var _a, _b;
        var value = valueRef.current;
        if (modelUri != null || value != null) {
            var existingModel_1 = modelUri != null ? monaco.editor.getModel(modelUri) : null;
            var model_1 = existingModel_1 !== null && existingModel_1 !== void 0 ? existingModel_1 : monaco.editor.createModel(value, monacoLanguage, modelUri);
            if (monacoLanguage != null && model_1.getLanguageId() !== monacoLanguage) {
                monaco.editor.setModelLanguage(model_1, monacoLanguage);
            }
            modelRef.current = model_1;
            (_a = editorRef.current) === null || _a === void 0 ? void 0 : _a.setModel(model_1);
            if (editorRef.current != null) {
                lastRestoreViewState(editorRef.current, model_1);
            }
            return function () {
                lastSaveViewState(editorRef.current, model_1);
                if (existingModel_1 == null) {
                    // Only dispose if we are the one that created the model
                    model_1.dispose();
                }
            };
        }
        else {
            modelRef.current = undefined;
            (_b = editorRef.current) === null || _b === void 0 ? void 0 : _b.setModel(null);
        }
    }, [monacoLanguage, modelUri, valueRef, lastSaveViewState, lastRestoreViewState]);
    // Create editor
    useEffect(function () {
        var containerElement = containerRef.current;
        if (containerElement != null) {
            var model = modelRef.current;
            var editor_1 = createEditor(containerElement, __assign(__assign({ model: model }, allOptions), { 
                // We need to override all IStandaloneEditorConstructionOptions fields to prevent conflicts with proper editor options (especially `language`)
                value: undefined, language: undefined, theme: undefined, autoDetectHighContrast: undefined, accessibilityHelpUrl: undefined, ariaContainerElement: undefined, dimension: undefined, overflowWidgetsDomNode: undefined }), overrideServices);
            editorRef.current = editor_1;
            if (model != null) {
                lastRestoreViewState(editor_1, model);
            }
            if (ref != null) {
                if (typeof ref === 'function') {
                    ref(editor_1);
                }
                else {
                    ref.current = editor_1;
                }
            }
            return function () {
                editor_1.dispose();
            };
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [overrideServices]);
    // Update value
    useEffect(function () {
        if (fixedCode != null) {
            var model = modelRef.current;
            var editor = editorRef.current;
            if (fixedCode !== model.getValue()) {
                preventTriggerChangeEventRef.current = true;
                console.debug('Replacing whole editor content');
                if (editor.getOption(monaco.editor.EditorOption.readOnly)) {
                    model.setValue(fixedCode);
                }
                else {
                    editor.pushUndoStop();
                    model.pushEditOperations([], [{
                            range: model.getFullModelRange(),
                            text: fixedCode
                        }], function () { return null; });
                    editor.pushUndoStop();
                }
                preventTriggerChangeEventRef.current = false;
            }
        }
    }, [fixedCode]);
    // Update options from props
    useEffect(function () {
        editorRef.current.updateOptions(allOptions);
    }, [allOptions]);
    // Keybindings
    useEffect(function () {
        var editor = editorRef.current;
        var disposable = updateEditorKeybindings(editor, keyBindingsMode, statusBarRef.current, keyBindings);
        return function () {
            disposable.dispose();
        };
    }, [keyBindings, keyBindingsMode]);
    // Update markers
    useEffect(function () {
        var model = modelRef.current;
        if (markers != null && model != null) {
            monaco.editor.setModelMarkers(model, 'customMarkers', markers);
            return function () {
                monaco.editor.setModelMarkers(model, 'customMarkers', []);
            };
        }
    }, [markers]);
    // Call onChange callback
    useEffect(function () {
        if (onChange != null) {
            var editor_2 = editorRef.current;
            var didChangeModelContentDisposable_1 = editor_2.onDidChangeModelContent(function (event) {
                if (!preventTriggerChangeEventRef.current) {
                    onChange(editor_2.getValue(), event);
                }
            });
            return function () {
                didChangeModelContentDisposable_1.dispose();
            };
        }
    }, [onChange]);
    useEffect(function () {
        if (onEditorOpenRequest != null) {
            var disposable_1 = registerEditorOpenHandler(function (model, input, source, sideBySide) { return __awaiter(_this, void 0, void 0, function () {
                return __generator(this, function (_a) {
                    if (source === editorRef.current) {
                        return [2 /*return*/, onEditorOpenRequest(model, input, source, sideBySide)];
                    }
                    return [2 /*return*/, null];
                });
            }); });
            return function () {
                disposable_1.dispose();
            };
        }
    }, [onEditorOpenRequest]);
    // Compute height
    useEffect(function () {
        if (requestedHeight !== 'auto') {
            setHeight(requestedHeight);
            return;
        }
        var editor = editorRef.current;
        var recomputeHeight = function () {
            var height = Math.max(minHeight, editor.getContentHeight() + STATUS_BAR_HEIGHT);
            if (maxHeight != null) {
                height = Math.min(height, maxHeight);
            }
            setHeight(height);
        };
        var onDidContentSizeChangeDisposable = editor.onDidContentSizeChange(debounce(recomputeHeight, 50));
        recomputeHeight();
        return function () {
            onDidContentSizeChangeDisposable.dispose();
        };
    }, [minHeight, maxHeight, requestedHeight]);
    var statusBarStyle = useMemo(function () {
        var _a, _b, _c, _d, _e, _f;
        return {
            backgroundColor: (_b = (_a = themeData === null || themeData === void 0 ? void 0 : themeData.getColor('statusBar.background')) === null || _a === void 0 ? void 0 : _a.toString()) !== null && _b !== void 0 ? _b : '#007ACC',
            color: (_d = (_c = themeData === null || themeData === void 0 ? void 0 : themeData.getColor('statusBar.foreground')) === null || _c === void 0 ? void 0 : _c.toString()) !== null && _d !== void 0 ? _d : '#FFFFFF',
            borderTop: "1px solid ".concat((_f = (_e = themeData === null || themeData === void 0 ? void 0 : themeData.getColor('statusBar.border')) === null || _e === void 0 ? void 0 : _e.toString()) !== null && _f !== void 0 ? _f : '#FFFFFF')
        };
    }, [themeData]);
    return (React.createElement("div", { className: 'react-monaco-editor-react-container', style: { height: height } },
        React.createElement("div", { className: 'react-monaco-editor-react-wrapper' },
            React.createElement("div", { className: 'react-monaco-editor-react', ref: containerRef }),
            React.createElement("div", { className: 'react-monaco-editor-react-status-bar', ref: statusBarRef, style: statusBarStyle }))));
}
export default forwardRef(MonacoEditor);
//# sourceMappingURL=MonacoEditor.js.map