import { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import * as monaco from 'monaco-editor';
import deepEqual from 'deep-equal';
import { getConfiguration, getThemeData, onConfigurationChanged } from '@codingame/monaco-editor-wrapper';
var standaloneThemeService = monaco.editor.StaticServices.standaloneThemeService.get();
export function useThemeData() {
    var _a = useState(standaloneThemeService.getColorTheme().themeName), themeName = _a[0], setThemeName = _a[1];
    var updateTheme = function (theme) {
        setThemeName(theme.themeName);
    };
    useEffect(function () {
        var disposable = standaloneThemeService.onDidColorThemeChange(updateTheme);
        updateTheme(standaloneThemeService.getColorTheme());
        return function () {
            disposable.dispose();
        };
    }, []);
    return useMemo(function () {
        return getThemeData(themeName);
    }, [themeName]);
}
export function useUserConfiguration(programmingLanguageId) {
    var _a = useState(function () { return getConfiguration(); }), userConfiguration = _a[0], setUserConfiguration = _a[1];
    useEffect(function () {
        var updateOptions = function () {
            setUserConfiguration(getConfiguration(programmingLanguageId));
        };
        var configurationChangeDisposable = onConfigurationChanged(updateOptions);
        return function () {
            configurationChangeDisposable.dispose();
        };
    }, [programmingLanguageId]);
    return userConfiguration;
}
export function useLastValueRef(value) {
    var ref = useRef(value);
    ref.current = value;
    return ref;
}
export function useLastVersion(func) {
    var ref = useRef(func);
    useEffect(function () {
        ref.current = func;
    }, [func]);
    return useCallback(function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return ref.current.apply(ref, args);
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
}
export function usePrevious(value) {
    var ref = useRef();
    useEffect(function () {
        ref.current = value;
    }, [value]);
    return ref.current;
}
export function useDeepMemo(memoFn, deps, isEqual) {
    if (isEqual === void 0) { isEqual = deepEqual; }
    var ref = useRef();
    if (ref.current == null || !isEqual(deps, ref.current.deps)) {
        ref.current = { deps: deps, value: memoFn() };
    }
    return ref.current.value;
}
//# sourceMappingURL=hooks.js.map